package sun.security.jgss.krb5;

import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import javax.security.auth.kerberos.KeyTab;
import sun.security.krb5.PrincipalName;
import sun.security.krb5.EncryptionKey;
import sun.security.krb5.KerberosSecrets;
import javax.security.auth.kerberos.KerberosPrincipal;
import java.io.IOException;
import sun.security.krb5.KrbException;
import sun.security.krb5.Credentials;
import javax.security.auth.login.LoginException;
import sun.security.jgss.GSSUtil;
import javax.security.auth.Subject;
import javax.security.auth.kerberos.KerberosTicket;
import java.security.AccessControlContext;
import sun.security.jgss.GSSCaller;

public class Krb5Util
{
    static final boolean DEBUG;
    
    private Krb5Util() {
    }
    
    public static KerberosTicket getTicketFromSubjectAndTgs(GSSCaller gssCaller, String s, String service, String s2, AccessControlContext acc) throws LoginException, KrbException, IOException {
        Subject subject = Subject.getSubject(acc);
        KerberosTicket credsToTicket = (KerberosTicket)SubjectComber.find(subject, service, s, KerberosTicket.class);
        if (credsToTicket != null) {
            return credsToTicket;
        }
        Subject login = null;
        if (!GSSUtil.useSubjectCredsOnly(gssCaller)) {
            try {
                login = GSSUtil.login(gssCaller, GSSUtil.GSS_KRB5_MECH_OID);
                credsToTicket = (KerberosTicket)SubjectComber.find(login, service, s, KerberosTicket.class);
                if (credsToTicket != null) {
                    return credsToTicket;
                }
            }
            catch (LoginException ex) {}
        }
        KerberosTicket kerberosTicket = (KerberosTicket)SubjectComber.find(subject, s2, s, KerberosTicket.class);
        int n;
        if (kerberosTicket == null && login != null) {
            kerberosTicket = (KerberosTicket)SubjectComber.find(login, s2, s, KerberosTicket.class);
            n = false;
        }
        else {
            n = true;
        }
        if (kerberosTicket != null) {
            Credentials acquireServiceCreds = Credentials.acquireServiceCreds(service, ticketToCreds(kerberosTicket));
            if (acquireServiceCreds != null) {
                credsToTicket = credsToTicket(acquireServiceCreds);
                if (n && subject != null && !subject.isReadOnly()) {
                    subject.getPrivateCredentials().add(credsToTicket);
                }
            }
        }
        return credsToTicket;
    }
    
    static KerberosTicket getServiceTicket(GSSCaller gssCaller, String s, String s2, AccessControlContext acc) throws LoginException {
        return (KerberosTicket)SubjectComber.find(Subject.getSubject(acc), s2, s, KerberosTicket.class);
    }
    
    static KerberosTicket getInitialTicket(GSSCaller gssCaller, String s, AccessControlContext acc) throws LoginException {
        KerberosTicket kerberosTicket = (KerberosTicket)SubjectComber.find(Subject.getSubject(acc), null, s, KerberosTicket.class);
        if (kerberosTicket == null && !GSSUtil.useSubjectCredsOnly(gssCaller)) {
            kerberosTicket = (KerberosTicket)SubjectComber.find(GSSUtil.login(gssCaller, GSSUtil.GSS_KRB5_MECH_OID), null, s, KerberosTicket.class);
            return kerberosTicket;
        }
        return kerberosTicket;
    }
    
    public static Subject getSubject(GSSCaller gssCaller, AccessControlContext acc) throws LoginException {
        Subject subject = Subject.getSubject(acc);
        if (subject == null && !GSSUtil.useSubjectCredsOnly(gssCaller)) {
            subject = GSSUtil.login(gssCaller, GSSUtil.GSS_KRB5_MECH_OID);
            return subject;
        }
        return subject;
    }
    
    public static ServiceCreds getServiceCreds(GSSCaller gssCaller, String s, AccessControlContext acc) throws LoginException {
        Subject subject = Subject.getSubject(acc);
        ServiceCreds serviceCreds = null;
        if (subject != null) {
            serviceCreds = ServiceCreds.getInstance(subject, s);
        }
        if (serviceCreds == null && !GSSUtil.useSubjectCredsOnly(gssCaller)) {
            serviceCreds = ServiceCreds.getInstance(GSSUtil.login(gssCaller, GSSUtil.GSS_KRB5_MECH_OID), s);
            return serviceCreds;
        }
        return serviceCreds;
    }
    
    public static KerberosTicket credsToTicket(Credentials credentials) {
        EncryptionKey sessionKey = credentials.getSessionKey();
        KerberosTicket kerberosTicket = new KerberosTicket(credentials.getEncoded(), new KerberosPrincipal(credentials.getClient().getName()), new KerberosPrincipal(credentials.getServer().getName(), 2), sessionKey.getBytes(), sessionKey.getEType(), credentials.getFlags(), credentials.getAuthTime(), credentials.getStartTime(), credentials.getEndTime(), credentials.getRenewTill(), credentials.getClientAddresses());
        PrincipalName clientAlias = credentials.getClientAlias();
        PrincipalName serverAlias = credentials.getServerAlias();
        if (clientAlias != null) {
            KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketSetClientAlias(kerberosTicket, new KerberosPrincipal(clientAlias.getName(), clientAlias.getNameType()));
        }
        if (serverAlias != null) {
            KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketSetServerAlias(kerberosTicket, new KerberosPrincipal(serverAlias.getName(), serverAlias.getNameType()));
            return kerberosTicket;
        }
        return kerberosTicket;
    }
    
    public static Credentials ticketToCreds(KerberosTicket kerberosTicket) throws KrbException, IOException {
        KerberosPrincipal kerberosTicketGetClientAlias = KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketGetClientAlias(kerberosTicket);
        KerberosPrincipal kerberosTicketGetServerAlias = KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketGetServerAlias(kerberosTicket);
        return new Credentials(kerberosTicket.getEncoded(), kerberosTicket.getClient().getName(), (kerberosTicketGetClientAlias != null) ? kerberosTicketGetClientAlias.getName() : null, kerberosTicket.getServer().getName(), (kerberosTicketGetServerAlias != null) ? kerberosTicketGetServerAlias.getName() : null, kerberosTicket.getSessionKey().getEncoded(), kerberosTicket.getSessionKeyType(), kerberosTicket.getFlags(), kerberosTicket.getAuthTime(), kerberosTicket.getStartTime(), kerberosTicket.getEndTime(), kerberosTicket.getRenewTill(), kerberosTicket.getClientAddresses());
    }
    
    public static sun.security.krb5.internal.ktab.KeyTab snapshotFromJavaxKeyTab(KeyTab keyTab) {
        return KerberosSecrets.getJavaxSecurityAuthKerberosAccess().keyTabTakeSnapshot(keyTab);
    }
    
    public static EncryptionKey[] keysFromJavaxKeyTab(KeyTab keyTab, PrincipalName service) {
        return snapshotFromJavaxKeyTab(keyTab).readServiceKeys(service);
    }
    
    static {
        DEBUG = (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.security.krb5.debug"));
    }
}
